@@ -41,17 +41,10 @@
 //M*/
 
 #include "test_precomp.hpp"
+#include "opencv2/ts/ocl_test.hpp" // T-API like tests
 
-#include <string>
-#include <iostream>
-#include <fstream>
-#include <iterator>
-#include <limits>
-#include <numeric>
-
-using namespace cv;
-using namespace std;
-
+namespace opencv_test {
+namespace {
 
 class CV_OperationsTest : public cvtest::BaseTest
 {
@@ -63,7 +56,7 @@ protected:
 
     struct test_excep
     {
-        test_excep(const string& _s=string("")) : s(_s) {};
+        test_excep(const string& _s=string("")) : s(_s) { }
         string s;
     };
 
@@ -75,6 +68,8 @@ protected:
     bool TestSparseMat();
     bool TestVec();
     bool TestMatxMultiplication();
+    bool TestMatxElementwiseDivison();
+    bool TestMatMatxCastSum();
     bool TestSubMatAccess();
     bool TestExp();
     bool TestSVD();
@@ -82,11 +77,11 @@ protected:
 
     void checkDiff(const Mat& m1, const Mat& m2, const string& s)
     {
-        if (norm(m1, m2, NORM_INF) != 0) throw test_excep(s);
+        if (cvtest::norm(m1, m2, NORM_INF) != 0) throw test_excep(s);
     }
     void checkDiffF(const Mat& m1, const Mat& m2, const string& s)
     {
-        if (norm(m1, m2, NORM_INF) > 1e-5) throw test_excep(s);
+        if (cvtest::norm(m1, m2, NORM_INF) > 1e-5) throw test_excep(s);
     }
 };
 
@@ -111,7 +106,7 @@ CV_OperationsTest::~CV_OperationsTest() {}
 template<typename _Tp> void CV_OperationsTest::TestType(Size sz, _Tp value)
 {
     cv::Mat_<_Tp> m(sz);
-    CV_Assert(m.cols == sz.width && m.rows == sz.height && m.depth() == DataType<_Tp>::depth &&
+    CV_Assert(m.cols == sz.width && m.rows == sz.height && m.depth() == cv::traits::Depth<_Tp>::value &&
               m.channels() == DataType<_Tp>::channels &&
               m.elemSize() == sizeof(_Tp) && m.step == m.elemSize()*m.cols);
     for( int y = 0; y < sz.height; y++ )
@@ -140,16 +135,16 @@ bool CV_OperationsTest::TestMat()
         res = Mat(Mat(2 * rot_2x3) * res - shi_2x1) + shift;
 
         Mat tmp, res2;
-        add(one_3x1, shi_3x1, tmp);
-        add(tmp, shi_3x1, tmp);
-        add(tmp, shi_3x1, tmp);
-        gemm(rot_2x3, tmp, 2, shi_2x1, -1, res2, 0);
-        add(res2, Mat(2, 1, CV_32F, shift), res2);
+        cv::add(one_3x1, shi_3x1, tmp);
+        cv::add(tmp, shi_3x1, tmp);
+        cv::add(tmp, shi_3x1, tmp);
+        cv::gemm(rot_2x3, tmp, 2, shi_2x1, -1, res2, 0);
+        cv::add(res2, Mat(2, 1, CV_32F, shift), res2);
 
         CHECK_DIFF(res, res2);
 
         Mat mat4x4(4, 4, CV_32F);
-        randu(mat4x4, Scalar(0), Scalar(10));
+        cv::randu(mat4x4, Scalar(0), Scalar(10));
 
         Mat roi1 = mat4x4(Rect(Point(1, 1), Size(2, 2)));
         Mat roi2 = mat4x4(Range(1, 3), Range(1, 3));
@@ -472,7 +467,7 @@ bool CV_OperationsTest::TestSubMatAccess()
         Vec3f ydir(1.f, 0.f, 1.f);
         Vec3f fpt(0.1f, 0.7f, 0.2f);
         T_bs.setTo(0);
-        T_bs(Range(0,3),Range(2,3)) = 1.0*Mat(cdir); // wierd OpenCV stuff, need to do multiply
+        T_bs(Range(0,3),Range(2,3)) = 1.0*Mat(cdir); // weird OpenCV stuff, need to do multiply
         T_bs(Range(0,3),Range(1,2)) = 1.0*Mat(ydir);
         T_bs(Range(0,3),Range(0,1)) = 1.0*Mat(cdir.cross(ydir));
         T_bs(Range(0,3),Range(3,4)) = 1.0*Mat(fpt);
@@ -487,7 +482,7 @@ bool CV_OperationsTest::TestSubMatAccess()
             coords.push_back(T_bs(i));
             //std::cout << T_bs1(i) << std::endl;
         }
-        CV_Assert( norm(coords, T_bs.reshape(1,1), NORM_INF) == 0 );
+        CV_Assert( cvtest::norm(coords, T_bs.reshape(1,1), NORM_INF) == 0 );
     }
     catch (const test_excep& e)
     {
@@ -514,19 +509,19 @@ bool CV_OperationsTest::TestTemplateMat()
         Mat_<float> resS = rot_2x3 * one_3x1;
 
         Mat_<float> tmp, res2, resS2;
-        add(one_3x1, shi_3x1, tmp);
-        add(tmp, shi_3x1, tmp);
-        add(tmp, shi_3x1, tmp);
-        gemm(rot_2x3, tmp, 2, shi_2x1, -1, res2, 0);
-        add(res2, Mat(2, 1, CV_32F, shift), res2);
+        cv::add(one_3x1, shi_3x1, tmp);
+        cv::add(tmp, shi_3x1, tmp);
+        cv::add(tmp, shi_3x1, tmp);
+        cv::gemm(rot_2x3, tmp, 2, shi_2x1, -1, res2, 0);
+        cv::add(res2, Mat(2, 1, CV_32F, shift), res2);
 
-        gemm(rot_2x3, one_3x1, 1, shi_2x1, 0, resS2, 0);
+        cv::gemm(rot_2x3, one_3x1, 1, shi_2x1, 0, resS2, 0);
         CHECK_DIFF(res, res2);
         CHECK_DIFF(resS, resS2);
 
 
         Mat_<float> mat4x4(4, 4);
-        randu(mat4x4, Scalar(0), Scalar(10));
+        cv::randu(mat4x4, Scalar(0), Scalar(10));
 
         Mat_<float> roi1 = mat4x4(Rect(Point(1, 1), Size(2, 2)));
         Mat_<float> roi2 = mat4x4(Range(1, 3), Range(1, 3));
@@ -775,14 +770,14 @@ bool CV_OperationsTest::TestTemplateMat()
         mvf.push_back(Mat_<float>::zeros(4, 3));
         merge(mvf, mf2);
         split(mf2, mvf2);
-        CV_Assert( norm(mvf2[0], mvf[0], CV_C) == 0 &&
-                  norm(mvf2[1], mvf[1], CV_C) == 0 );
+        CV_Assert( cvtest::norm(mvf2[0], mvf[0], CV_C) == 0 &&
+                  cvtest::norm(mvf2[1], mvf[1], CV_C) == 0 );
 
         {
         Mat a(2,2,CV_32F,1.f);
         Mat b(1,2,CV_32F,1.f);
         Mat c = (a*b.t()).t();
-        CV_Assert( norm(c, CV_L1) == 4. );
+        CV_Assert( cvtest::norm(c, CV_L1) == 4. );
         }
 
         bool badarg_catched = false;
@@ -800,13 +795,13 @@ bool CV_OperationsTest::TestTemplateMat()
 
         Size size(2, 5);
         TestType<float>(size, 1.f);
-        cv::Vec3f val1 = 1.f;
+        cv::Vec3f val1(1.f);
         TestType<cv::Vec3f>(size, val1);
-        cv::Matx31f val2 = 1.f;
+        cv::Matx31f val2(1.f);
         TestType<cv::Matx31f>(size, val2);
-        cv::Matx41f val3 = 1.f;
+        cv::Matx41f val3(1.f);
         TestType<cv::Matx41f>(size, val3);
-        cv::Matx32f val4 = 1.f;
+        cv::Matx32f val4(1.f);
         TestType<cv::Matx32f>(size, val4);
     }
     catch (const test_excep& e)
@@ -891,6 +886,96 @@ bool CV_OperationsTest::TestMatxMultiplication()
     return true;
 }
 
+bool CV_OperationsTest::TestMatMatxCastSum()
+{
+    try
+    {
+        Mat ref1 = (Mat_<double>(3, 1) << 1, 2, 3);
+        Mat ref2 = (Mat_<double>(3, 1) << 3, 4, 5);
+        Mat ref3 = Mat::ones(3, 1, CV_64FC1);
+
+        Mat mat = Mat::zeros(3, 1, CV_64FC1);
+
+        Mat tst1 = ref1.clone();
+        Mat_<double> tst2 = ref2.clone();
+        Matx<double, 3, 1> tst3(1, 2, 3);
+        Vec3d tst4(3, 4, 5);
+        Scalar tst5(1, 2, 3);
+        Mat res;
+
+        res = mat + tst1;
+        CHECK_DIFF_FLT(res, ref1);
+        res = mat + tst2;
+        CHECK_DIFF_FLT(res, ref2);
+        res = mat + tst3;
+        CHECK_DIFF_FLT(res, ref1);
+        res = mat + tst4;
+        CHECK_DIFF_FLT(res, ref2);
+
+        res = mat + tst5;
+        CHECK_DIFF_FLT(res, ref3);
+        res = mat + 1;
+        CHECK_DIFF_FLT(res, ref3);
+
+        cv::add(mat, tst1, res);
+        CHECK_DIFF_FLT(res, ref1);
+        cv::add(mat, tst2, res);
+        CHECK_DIFF_FLT(res, ref2);
+        cv::add(mat, tst3, res);
+        CHECK_DIFF_FLT(res, ref1);
+        cv::add(mat, tst4, res);
+        CHECK_DIFF_FLT(res, ref2);
+
+        cv::add(mat, tst5, res);
+        CHECK_DIFF_FLT(res, ref3);
+        cv::add(mat, 1, res);
+        CHECK_DIFF_FLT(res, ref3);
+
+        res = mat.clone(); res += tst1;
+        CHECK_DIFF_FLT(res, ref1);
+        res = mat.clone(); res += tst2;
+        CHECK_DIFF_FLT(res, ref2);
+        res = mat.clone(); res += tst3;
+        CHECK_DIFF_FLT(res, ref1);
+        res = mat.clone(); res += tst4;
+        CHECK_DIFF_FLT(res, ref2);
+
+        res = mat.clone(); res += tst5;
+        CHECK_DIFF_FLT(res, ref3);
+        res = mat.clone(); res += 1;
+        CHECK_DIFF_FLT(res, ref3);
+    }
+    catch (const test_excep& e)
+    {
+        ts->printf(cvtest::TS::LOG, "%s\n", e.s.c_str());
+        ts->set_failed_test_info(cvtest::TS::FAIL_MISMATCH);
+        return false;
+    }
+    return true;
+}
+
+bool CV_OperationsTest::TestMatxElementwiseDivison()
+{
+    try
+    {
+        Matx22f mat(2, 4, 6, 8);
+        Matx22f mat2(2, 2, 2, 2);
+
+        Matx22f res = mat.div(mat2);
+
+        if(res(0, 0) != 1.0) throw test_excep();
+        if(res(0, 1) != 2.0) throw test_excep();
+        if(res(1, 0) != 3.0) throw test_excep();
+        if(res(1, 1) != 4.0) throw test_excep();
+    }
+    catch(const test_excep&)
+    {
+        ts->set_failed_test_info(cvtest::TS::FAIL_INVALID_OUTPUT);
+        return false;
+    }
+    return true;
+}
+
 
 bool CV_OperationsTest::TestVec()
 {
@@ -954,7 +1039,14 @@ bool CV_OperationsTest::operations1()
         Size sz(10, 20);
         if (sz.area() != 200) throw test_excep();
         if (sz.width != 10 || sz.height != 20) throw test_excep();
-        if (((CvSize)sz).width != 10 || ((CvSize)sz).height != 20) throw test_excep();
+        if (cvSize(sz).width != 10 || cvSize(sz).height != 20) throw test_excep();
+
+        Rect r1(0, 0, 10, 20);
+        Size sz1(5, 10);
+        r1 -= sz1;
+        if (r1.size().width != 5 || r1.size().height != 10) throw test_excep();
+        Rect r2 = r1 - sz1;
+        if (r2.size().width != 0 || r2.size().height != 0) throw test_excep();
 
         Vec<double, 5> v5d(1, 1, 1, 1, 1);
         Vec<double, 6> v6d(1, 1, 1, 1, 1, 1);
@@ -972,17 +1064,17 @@ bool CV_OperationsTest::operations1()
         Mat A(1, 32, CV_32F), B;
         for( int i = 0; i < A.cols; i++ )
             A.at<float>(i) = (float)(i <= 12 ? i : 24 - i);
-        transpose(A, B);
+        cv::transpose(A, B);
 
         int minidx[2] = {0, 0}, maxidx[2] = {0, 0};
         double minval = 0, maxval = 0;
-        minMaxIdx(A, &minval, &maxval, minidx, maxidx);
+        cv::minMaxIdx(A, &minval, &maxval, minidx, maxidx);
 
         if( !(minidx[0] == 0 && minidx[1] == 31 && maxidx[0] == 0 && maxidx[1] == 12 &&
                   minval == -7 && maxval == 12))
             throw test_excep();
 
-        minMaxIdx(B, &minval, &maxval, minidx, maxidx);
+        cv::minMaxIdx(B, &minval, &maxval, minidx, maxidx);
 
         if( !(minidx[0] == 31 && minidx[1] == 0 && maxidx[0] == 12 && maxidx[1] == 0 &&
               minval == -7 && maxval == 12))
@@ -990,14 +1082,14 @@ bool CV_OperationsTest::operations1()
 
         Matx33f b(1.f, 2.f, 3.f, 4.f, 5.f, 6.f, 7.f, 8.f, 9.f);
         Mat c;
-        add(Mat::zeros(3, 3, CV_32F), b, c);
-        CV_Assert( norm(b, c, CV_C) == 0 );
+        cv::add(Mat::zeros(3, 3, CV_32F), b, c);
+        CV_Assert( cvtest::norm(b, c, CV_C) == 0 );
 
-        add(Mat::zeros(3, 3, CV_64F), b, c, noArray(), c.type());
-        CV_Assert( norm(b, c, CV_C) == 0 );
+        cv::add(Mat::zeros(3, 3, CV_64F), b, c, noArray(), c.type());
+        CV_Assert( cvtest::norm(b, c, CV_C) == 0 );
 
-        add(Mat::zeros(6, 1, CV_64F), 1, c, noArray(), c.type());
-        CV_Assert( norm(Matx61f(1.f, 1.f, 1.f, 1.f, 1.f, 1.f), c, CV_C) == 0 );
+        cv::add(Mat::zeros(6, 1, CV_64F), 1, c, noArray(), c.type());
+        CV_Assert( cvtest::norm(Matx61f(1.f, 1.f, 1.f, 1.f, 1.f, 1.f), c, CV_C) == 0 );
 
         vector<Point2f> pt2d(3);
         vector<Point3d> pt3d(2);
@@ -1009,7 +1101,7 @@ bool CV_OperationsTest::operations1()
                 0.9058f, 0.0975f, 0.9649f, 0.4854f,
                 0.1270f, 0.2785f, 0.1576f, 0.8003f,
                 0.9134f, 0.5469f, 0.9706f, 0.1419f);
-        double d = determinant(m44);
+        double d = cv::determinant(m44);
         CV_Assert( fabs(d - (-0.0262)) <= 0.001 );
 
         Cv32suf z;
@@ -1043,11 +1135,11 @@ bool CV_OperationsTest::TestSVD()
         Mat A = (Mat_<double>(3,4) << 1, 2, -1, 4, 2, 4, 3, 5, -1, -2, 6, 7);
         Mat x;
         SVD::solveZ(A,x);
-        if( norm(A*x, CV_C) > FLT_EPSILON )
+        if( cvtest::norm(A*x, CV_C) > FLT_EPSILON )
             throw test_excep();
 
         SVD svd(A, SVD::FULL_UV);
-        if( norm(A*svd.vt.row(3).t(), CV_C) > FLT_EPSILON )
+        if( cvtest::norm(A*svd.vt.row(3).t(), CV_C) > FLT_EPSILON )
             throw test_excep();
 
         Mat Dp(3,3,CV_32FC1);
@@ -1071,11 +1163,11 @@ bool CV_OperationsTest::TestSVD()
         W=decomp.w;
         Mat I = Mat::eye(3, 3, CV_32F);
 
-        if( norm(U*U.t(), I, CV_C) > FLT_EPSILON ||
-            norm(Vt*Vt.t(), I, CV_C) > FLT_EPSILON ||
+        if( cvtest::norm(U*U.t(), I, CV_C) > FLT_EPSILON ||
+            cvtest::norm(Vt*Vt.t(), I, CV_C) > FLT_EPSILON ||
             W.at<float>(2) < 0 || W.at<float>(1) < W.at<float>(2) ||
             W.at<float>(0) < W.at<float>(1) ||
-            norm(U*Mat::diag(W)*Vt, Q, CV_C) > FLT_EPSILON )
+            cvtest::norm(U*Mat::diag(W)*Vt, Q, CV_C) > FLT_EPSILON )
             throw test_excep();
     }
     catch(const test_excep&)
@@ -1097,8 +1189,8 @@ void CV_OperationsTest::run( int /* start_from */)
     if (!TestTemplateMat())
         return;
 
- /*   if (!TestMatND())
-        return;*/
+    if (!TestMatND())
+        return;
 
     if (!TestSparseMat())
         return;
@@ -1109,6 +1201,12 @@ void CV_OperationsTest::run( int /* start_from */)
     if (!TestMatxMultiplication())
         return;
 
+    if (!TestMatxElementwiseDivison())
+        return;
+
+    if (!TestMatMatxCastSum())
+        return;
+
     if (!TestSubMatAccess())
         return;
 
@@ -1214,3 +1312,174 @@ protected:
 };
 
 TEST(Core_SparseMat, iterations) { CV_SparseMatTest test; test.safe_run(); }
+
+TEST(MatTestRoi, adjustRoiOverflow)
+{
+    Mat m(15, 10, CV_32S);
+    Mat roi(m, cv::Range(2, 10), cv::Range(3,6));
+    int rowsInROI = roi.rows;
+    roi.adjustROI(1, 0, 0, 0);
+
+    ASSERT_EQ(roi.rows, rowsInROI + 1);
+
+    roi.adjustROI(-m.rows, -m.rows, 0, 0);
+
+    ASSERT_EQ(roi.rows, m.rows);
+}
+
+
+CV_ENUM(SortRowCol, SORT_EVERY_COLUMN, SORT_EVERY_ROW)
+CV_ENUM(SortOrder, SORT_ASCENDING, SORT_DESCENDING)
+
+PARAM_TEST_CASE(sortIdx, MatDepth, SortRowCol, SortOrder, Size, bool)
+{
+    int type;
+    Size size;
+    int flags;
+    bool use_roi;
+
+    Mat src, src_roi;
+    Mat dst, dst_roi;
+
+    virtual void SetUp()
+    {
+        int depth = GET_PARAM(0);
+        int rowFlags = GET_PARAM(1);
+        int orderFlags = GET_PARAM(2);
+        size = GET_PARAM(3);
+        use_roi = GET_PARAM(4);
+
+        type = CV_MAKE_TYPE(depth, 1);
+
+        flags = rowFlags | orderFlags;
+    }
+
+    void generateTestData()
+    {
+        Border srcBorder = randomBorder(0, use_roi ? MAX_VALUE : 0);
+        randomSubMat(src, src_roi, size, srcBorder, type, -100, 100);
+
+        Border dstBorder = randomBorder(0, use_roi ? MAX_VALUE : 0);
+        randomSubMat(dst, dst_roi, size, dstBorder, CV_32S, 5, 16);
+    }
+
+    template<typename T>
+    void check_(const cv::Mat& values_, const cv::Mat_<int>& idx_)
+    {
+        cv::Mat_<T>& values = (cv::Mat_<T>&)values_;
+        cv::Mat_<int>& idx = (cv::Mat_<int>&)idx_;
+        size_t N = values.total();
+        std::vector<bool> processed(N, false);
+        int prevIdx = idx(0);
+        T prevValue = values(prevIdx);
+        processed[prevIdx] = true;
+        for (size_t i = 1; i < N; i++)
+        {
+            int nextIdx = idx((int)i);
+            T value = values(nextIdx);
+            ASSERT_EQ(false, processed[nextIdx]) << "Indexes must be unique. i=" << i << " idx=" << nextIdx << std::endl << idx;
+            processed[nextIdx] = true;
+            if ((flags & SORT_DESCENDING) == SORT_DESCENDING)
+                ASSERT_GE(prevValue, value) << "i=" << i << " prevIdx=" << prevIdx << " idx=" << nextIdx;
+            else
+                ASSERT_LE(prevValue, value) << "i=" << i << " prevIdx=" << prevIdx << " idx=" << nextIdx;
+            prevValue = value;
+            prevIdx = nextIdx;
+        }
+    }
+
+    void validate()
+    {
+        ASSERT_EQ(CV_32SC1, dst_roi.type());
+        ASSERT_EQ(size, dst_roi.size());
+        bool isColumn = (flags & SORT_EVERY_COLUMN) == SORT_EVERY_COLUMN;
+        size_t N = isColumn ? src_roi.cols : src_roi.rows;
+        Mat values_row((int)N, 1, type), idx_row((int)N, 1, CV_32S);
+        for (size_t i = 0; i < N; i++)
+        {
+            SCOPED_TRACE(cv::format("row/col=%d", (int)i));
+            if (isColumn)
+            {
+                src_roi.col((int)i).copyTo(values_row);
+                dst_roi.col((int)i).copyTo(idx_row);
+            }
+            else
+            {
+                src_roi.row((int)i).copyTo(values_row);
+                dst_roi.row((int)i).copyTo(idx_row);
+            }
+            switch(type)
+            {
+            case CV_8U: check_<uchar>(values_row, idx_row); break;
+            case CV_8S: check_<char>(values_row, idx_row); break;
+            case CV_16S: check_<short>(values_row, idx_row); break;
+            case CV_32S: check_<int>(values_row, idx_row); break;
+            case CV_32F: check_<float>(values_row, idx_row); break;
+            case CV_64F: check_<double>(values_row, idx_row); break;
+            default: ASSERT_FALSE(true) << "Unsupported type: " << type;
+            }
+        }
+    }
+};
+
+TEST_P(sortIdx, simple)
+{
+    for (int j = 0; j < 5; j++)
+    {
+        generateTestData();
+
+        cv::sortIdx(src_roi, dst_roi, flags);
+        validate();
+    }
+}
+
+INSTANTIATE_TEST_CASE_P(Core, sortIdx, Combine(
+        Values(CV_8U, CV_8S, CV_16S, CV_32S, CV_32F, CV_64F), // depth
+        Values(SORT_EVERY_COLUMN, SORT_EVERY_ROW),
+        Values(SORT_ASCENDING, SORT_DESCENDING),
+        Values(Size(3, 3), Size(16, 8)),
+        ::testing::Bool()
+));
+
+
+TEST(Core_sortIdx, regression_8941)
+{
+    cv::Mat src = (cv::Mat_<int>(3, 3) <<
+        1, 2, 3,
+        0, 9, 5,
+        8, 1, 6
+    );
+    cv::Mat expected = (cv::Mat_<int>(3, 1) <<
+        1,
+        0,
+        2
+    );
+
+    cv::Mat result;
+    cv::sortIdx(src.col(0), result, CV_SORT_EVERY_COLUMN | CV_SORT_ASCENDING);
+#if 0
+    std::cout << src.col(0) << std::endl;
+    std::cout << result << std::endl;
+#endif
+    ASSERT_EQ(expected.size(), result.size());
+    EXPECT_EQ(0, cvtest::norm(expected, result, NORM_INF)) <<
+        "result=" << std::endl << result << std::endl <<
+        "expected=" << std::endl << expected;
+}
+
+
+//These tests guard regressions against running MatExpr
+//operations on empty operands and giving bogus
+//results.
+TEST(Core_MatExpr, empty_check_15760)
+{
+    EXPECT_THROW(Mat c = min(Mat(), Mat()), cv::Exception);
+    EXPECT_THROW(Mat c = abs(Mat()), cv::Exception);
+    EXPECT_THROW(Mat c = min(Mat(), Mat()), cv::Exception);
+    EXPECT_THROW(Mat c = Mat() | Mat(), cv::Exception);
+    EXPECT_THROW(Mat c = Mat() + Mat(), cv::Exception);
+    EXPECT_THROW(Mat c = Mat().t(), cv::Exception);
+    EXPECT_THROW(Mat c = Mat().cross(Mat()), cv::Exception);
+}
+
+}} // namespace
