@@ -21,6 +21,54 @@ namespace cv { namespace dnn {
 
 #ifdef HAVE_INF_ENGINE
 
+static Backend parseInferenceEngineBackendType(const cv::String& backend)
+{
+    CV_Assert(!backend.empty());
+    if (backend == CV_DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
+        return DNN_BACKEND_INFERENCE_ENGINE_NGRAPH;
+    if (backend == CV_DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_API)
+        return DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019;
+    CV_Error(Error::StsBadArg, cv::format("Unknown IE backend: %s", backend.c_str()));
+}
+static const char* dumpInferenceEngineBackendType(Backend backend)
+{
+    if (backend == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
+        return CV_DNN_BACKEND_INFERENCE_ENGINE_NGRAPH;
+    if (backend == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019)
+        return CV_DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_API;
+    CV_Error(Error::StsBadArg, cv::format("Invalid backend ID for IE: %d", backend));
+}
+Backend& getInferenceEngineBackendTypeParam()
+{
+    static Backend param = parseInferenceEngineBackendType(
+        utils::getConfigurationParameterString("OPENCV_DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019_TYPE",
+#ifdef HAVE_NGRAPH
+            CV_DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_API  // future: CV_DNN_BACKEND_INFERENCE_ENGINE_NGRAPH
+#else
+            CV_DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_API
+#endif
+        )
+    );
+    return param;
+}
+
+CV__DNN_INLINE_NS_BEGIN
+
+cv::String getInferenceEngineBackendType()
+{
+    return dumpInferenceEngineBackendType(getInferenceEngineBackendTypeParam());
+}
+cv::String setInferenceEngineBackendType(const cv::String& newBackendType)
+{
+    Backend newBackend = parseInferenceEngineBackendType(newBackendType);
+    Backend& param = getInferenceEngineBackendTypeParam();
+    Backend old = param;
+    param = newBackend;
+    return dumpInferenceEngineBackendType(old);
+}
+
+CV__DNN_INLINE_NS_END
+
 // For networks with input layer which has an empty name, IE generates a name id[some_number].
 // OpenCV lets users use an empty input name and to prevent unexpected naming,
 // we can use some predefined name.
@@ -161,38 +209,25 @@ private:
     InferenceEngine::CNNLayer cnnLayer;
 };
 
-class InfEngineExtension : public InferenceEngine::IExtension
+InferenceEngine::StatusCode InfEngineExtension::getFactoryFor(
+        InferenceEngine::ILayerImplFactory*& factory,
+        const InferenceEngine::CNNLayer* cnnLayer,
+        InferenceEngine::ResponseDesc* resp
+) noexcept
 {
-public:
-    virtual void SetLogCallback(InferenceEngine::IErrorListener&) noexcept {}
-    virtual void Unload() noexcept {}
-    virtual void Release() noexcept {}
-    virtual void GetVersion(const InferenceEngine::Version*&) const noexcept {}
-
-    virtual InferenceEngine::StatusCode getPrimitiveTypes(char**&, unsigned int&,
-                                                          InferenceEngine::ResponseDesc*) noexcept
-    {
-        return InferenceEngine::StatusCode::OK;
-    }
-
-    InferenceEngine::StatusCode getFactoryFor(InferenceEngine::ILayerImplFactory*& factory,
-                                              const InferenceEngine::CNNLayer* cnnLayer,
-                                              InferenceEngine::ResponseDesc* resp) noexcept
-    {
-        if (cnnLayer->type != kOpenCVLayersType)
-            return InferenceEngine::StatusCode::NOT_IMPLEMENTED;
-        factory = new InfEngineCustomLayerFactory(cnnLayer);
-        return InferenceEngine::StatusCode::OK;
-    }
-};
+    if (cnnLayer->type != kOpenCVLayersType)
+        return InferenceEngine::StatusCode::NOT_IMPLEMENTED;
+    factory = new InfEngineCustomLayerFactory(cnnLayer);
+    return InferenceEngine::StatusCode::OK;
+}
 
 InfEngineBackendNode::InfEngineBackendNode(const InferenceEngine::Builder::Layer& _layer)
-    : BackendNode(DNN_BACKEND_INFERENCE_ENGINE), layer(_layer) {}
+    : BackendNode(DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019), layer(_layer) {}
 
     InfEngineBackendNode::InfEngineBackendNode(Ptr<Layer>& cvLayer_, std::vector<Mat*>& inputs,
                                                std::vector<Mat>& outputs,
                                                std::vector<Mat>& internals)
-        : BackendNode(DNN_BACKEND_INFERENCE_ENGINE), layer(cvLayer_->name),
+        : BackendNode(DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019), layer(cvLayer_->name),
           cvLayer(cvLayer_)
 {
     CV_Assert(!cvLayer->name.empty());
@@ -243,11 +278,28 @@ void InfEngineBackendNet::connect(const std::vector<Ptr<BackendWrapper> >& input
     {
         const auto& inp = inpWrappers[i];
         const std::string& inpName = inp->dataPtr->getName();
+
+        std::string inpLayerName = inpName;
+        size_t inpPortId = inpName.rfind('.');
+        if (inpPortId != std::string::npos)
+        {
+            std::string portIdStr = inpName.substr(inpPortId + 1);
+            if (std::all_of(portIdStr.begin(), portIdStr.end(), ::isdigit))
+            {
+                inpLayerName = inpName.substr(0, inpPortId);
+                inpPortId = atoi(portIdStr.c_str());
+            }
+            else
+                inpPortId = 0;
+        }
+        else
+            inpPortId = 0;
+
         int inpId;
-        it = layers.find(inpName);
+        it = layers.find(inpLayerName);
         if (it == layers.end())
         {
-            InferenceEngine::Builder::InputLayer inpLayer(!inpName.empty() ? inpName : kDefaultInpLayerName);
+            InferenceEngine::Builder::InputLayer inpLayer(!inpLayerName.empty() ? inpLayerName : kDefaultInpLayerName);
             std::vector<size_t> shape(inp->blob->getTensorDesc().getDims());
             inpLayer.setPort(InferenceEngine::Port(shape));
             inpId = netBuilder.addLayer(inpLayer);
@@ -257,24 +309,28 @@ void InfEngineBackendNet::connect(const std::vector<Ptr<BackendWrapper> >& input
         else
             inpId = it->second;
 
-        netBuilder.connect((size_t)inpId, {(size_t)layerId, i});
-        unconnectedLayersIds.erase(inpId);
+        netBuilder.connect({(size_t)inpId, inpPortId}, {(size_t)layerId, i});
+        unconnectedPorts.erase({inpId, inpPortId});
     }
     CV_Assert(!outputs.empty());
-    InferenceEngine::DataPtr dataPtr = infEngineDataNode(outputs[0]);
+    for (int i = 0; i < outputs.size(); ++i)
+    {
+        InferenceEngine::DataPtr dataPtr = infEngineDataNode(outputs[i]);
+        std::string outputName = outputs.size() > 1 ? (layerName + "." + std::to_string(i)) : layerName;
 #if INF_ENGINE_VER_MAJOR_LE(INF_ENGINE_RELEASE_2019R1)
-    dataPtr->name = layerName;
+        dataPtr->name = outputName;
 #else
-    dataPtr->setName(layerName);
+        dataPtr->setName(outputName);
 #endif
+    }
 }
 
-void InfEngineBackendNet::init(int targetId)
+void InfEngineBackendNet::init(Target targetId)
 {
     if (!hasNetOwner)
     {
-        CV_Assert(!unconnectedLayersIds.empty());
-        for (int id : unconnectedLayersIds)
+        CV_Assert(!unconnectedPorts.empty());
+        for (const auto& port : unconnectedPorts)
         {
             InferenceEngine::Builder::OutputLayer outLayer("myconv1");
 #if INF_ENGINE_VER_MAJOR_GE(INF_ENGINE_RELEASE_2019R1)
@@ -285,7 +341,7 @@ void InfEngineBackendNet::init(int targetId)
                                            InferenceEngine::Precision::FP32;
             outLayer.setPort(InferenceEngine::Port({}, p));
 #endif
-            netBuilder.addLayer({InferenceEngine::PortInfo(id)}, outLayer);
+            netBuilder.addLayer({InferenceEngine::PortInfo(port.first, port.second)}, outLayer);
         }
         netBuilder.getContext().addShapeInferImpl(kOpenCVLayersType,
                             std::make_shared<InfEngineCustomLayerShapeInfer>());
@@ -374,8 +430,10 @@ void InfEngineBackendNet::addLayer(InferenceEngine::Builder::Layer& layer)
 
     int id = netBuilder.addLayer(layer);
     const std::string& layerName = layer.getName();
+
     CV_Assert(layers.insert({layerName, id}).second);
-    unconnectedLayersIds.insert(id);
+    for (int i = 0; i < layer.getOutputPorts().size(); ++i)
+        unconnectedPorts.insert({id, i});
 
 #if INF_ENGINE_VER_MAJOR_GE(INF_ENGINE_RELEASE_2019R1)
     // By default, all the weights are connected to last ports ids.
@@ -403,7 +461,7 @@ static InferenceEngine::Layout estimateLayout(const Mat& m)
 
 static InferenceEngine::DataPtr wrapToInfEngineDataNode(const Mat& m, const std::string& name = "")
 {
-    std::vector<size_t> shape(&m.size[0], &m.size[0] + m.dims);
+    std::vector<size_t> shape = getShape<size_t>(m);
     if (m.type() == CV_32F)
         return InferenceEngine::DataPtr(new InferenceEngine::Data(name,
                {InferenceEngine::Precision::FP32, shape, estimateLayout(m)}));
@@ -429,7 +487,7 @@ InferenceEngine::Blob::Ptr wrapToInfEngineBlob(const Mat& m, const std::vector<s
 
 InferenceEngine::Blob::Ptr wrapToInfEngineBlob(const Mat& m, InferenceEngine::Layout layout)
 {
-    std::vector<size_t> shape(&m.size[0], &m.size[0] + m.dims);
+    std::vector<size_t> shape = getShape<size_t>(m);
     return wrapToInfEngineBlob(m, shape, layout);
 }
 
@@ -461,14 +519,14 @@ InferenceEngine::DataPtr infEngineDataNode(const Ptr<BackendWrapper>& ptr)
 }
 
 InfEngineBackendWrapper::InfEngineBackendWrapper(int targetId, const cv::Mat& m)
-    : BackendWrapper(DNN_BACKEND_INFERENCE_ENGINE, targetId)
+    : BackendWrapper(DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019, targetId)
 {
     dataPtr = wrapToInfEngineDataNode(m);
     blob = wrapToInfEngineBlob(m, estimateLayout(m));
 }
 
 InfEngineBackendWrapper::InfEngineBackendWrapper(Ptr<BackendWrapper> wrapper)
-    : BackendWrapper(DNN_BACKEND_INFERENCE_ENGINE, wrapper->targetId)
+    : BackendWrapper(DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019, wrapper->targetId)
 {
     Ptr<InfEngineBackendWrapper> ieWrapper = wrapper.dynamicCast<InfEngineBackendWrapper>();
     CV_Assert(!ieWrapper.empty());
@@ -506,7 +564,7 @@ static std::map<std::string, InferenceEngine::InferenceEnginePluginPtr>& getShar
     return sharedPlugins;
 }
 #else
-static InferenceEngine::Core& getCore()
+InferenceEngine::Core& getCore()
 {
     static InferenceEngine::Core core;
     return core;
@@ -565,7 +623,11 @@ static bool detectMyriadX_()
 #else
     try
     {
+#if INF_ENGINE_VER_MAJOR_LE(INF_ENGINE_RELEASE_2019R3)
         auto netExec = getCore().LoadNetwork(cnn, "MYRIAD", {{"VPU_PLATFORM", "VPU_2480"}});
+#else
+        auto netExec = getCore().LoadNetwork(cnn, "MYRIAD", {{"VPU_MYRIAD_PLATFORM", "VPU_MYRIAD_2480"}});
+#endif
 #endif
         auto infRequest = netExec.CreateInferRequest();
     } catch(...) {
@@ -610,7 +672,7 @@ void InfEngineBackendNet::initPlugin(InferenceEngine::CNNNetwork& net)
             {
                 candidates.push_back(param_pluginPath);
             }
-
+#if INF_ENGINE_VER_MAJOR_LE(INF_ENGINE_RELEASE_2019R3)
             if (device_name == "CPU" || device_name == "FPGA")
             {
                 std::string suffixes[] = {"_avx2", "_sse4", ""};
@@ -633,6 +695,7 @@ void InfEngineBackendNet::initPlugin(InferenceEngine::CNNNetwork& net)
 #endif  // _WIN32
                 }
             }
+#endif
             bool found = false;
             for (size_t i = 0; i != candidates.size(); ++i)
             {
@@ -646,8 +709,6 @@ void InfEngineBackendNet::initPlugin(InferenceEngine::CNNNetwork& net)
                     enginePtr->AddExtension(extension, 0);
 #else
                     ie.AddExtension(extension, "CPU");
-                    // OpenCV fallbacks as extensions.
-                    ie.AddExtension(std::make_shared<InfEngineExtension>(), "CPU");
 #endif
                     CV_LOG_INFO(NULL, "DNN-IE: Loaded extension plugin: " << libName);
                     found = true;
@@ -660,6 +721,10 @@ void InfEngineBackendNet::initPlugin(InferenceEngine::CNNNetwork& net)
                 CV_LOG_WARNING(NULL, "DNN-IE: Can't load extension plugin (extra layers for some networks). Specify path via OPENCV_DNN_IE_EXTRA_PLUGIN_PATH parameter");
             }
             // Some of networks can work without a library of extra layers.
+#if INF_ENGINE_VER_MAJOR_GT(INF_ENGINE_RELEASE_2019R1)
+            // OpenCV fallbacks as extensions.
+            ie.AddExtension(std::make_shared<InfEngineExtension>(), "CPU");
+#endif
 #ifndef _WIN32
             // Limit the number of CPU threads.
 #if INF_ENGINE_VER_MAJOR_LE(INF_ENGINE_RELEASE_2019R1)
@@ -686,11 +751,16 @@ void InfEngineBackendNet::initPlugin(InferenceEngine::CNNNetwork& net)
             {
                 if (layer->type == kOpenCVLayersType)
                 {
-                    layer->affinity = "CPU";
                     isHetero = true;
+#if INF_ENGINE_VER_MAJOR_LT(INF_ENGINE_RELEASE_2019R3)
+                    // Not sure about lower versions but in 2019R3 we do not need this
+                    layer->affinity = "CPU";
                 }
                 else
+                {
                     layer->affinity = device_name;
+#endif
+                }
             }
         }
         if (isHetero)
@@ -701,7 +771,7 @@ void InfEngineBackendNet::initPlugin(InferenceEngine::CNNNetwork& net)
     }
     catch (const std::exception& ex)
     {
-        CV_Error(Error::StsAssert, format("Failed to initialize Inference Engine backend: %s", ex.what()));
+        CV_Error(Error::StsError, format("Failed to initialize Inference Engine backend (device = %s): %s", device_name.c_str(), ex.what()));
     }
 }
 
@@ -741,6 +811,7 @@ void InfEngineBackendNet::InfEngineReqWrapper::makePromises(const std::vector<Pt
 void InfEngineBackendNet::forward(const std::vector<Ptr<BackendWrapper> >& outBlobsWrappers,
                                   bool isAsync)
 {
+    CV_LOG_DEBUG(NULL, "InfEngineBackendNet::forward(" << (isAsync ? "async" : "sync") << ")");
     // Look for finished requests.
     Ptr<InfEngineReqWrapper> reqWrapper;
     for (auto& wrapper : infRequests)
@@ -788,6 +859,8 @@ void InfEngineBackendNet::forward(const std::vector<Ptr<BackendWrapper> >& outBl
         infRequestPtr->SetCompletionCallback(
             [](InferenceEngine::IInferRequest::Ptr request, InferenceEngine::StatusCode status)
             {
+                CV_LOG_DEBUG(NULL, "DNN(IE): completionCallback(" << (int)status << ")");
+
                 InfEngineReqWrapper* wrapper;
                 request->GetUserData((void**)&wrapper, 0);
                 CV_Assert(wrapper && "Internal error");
@@ -913,8 +986,9 @@ bool InfEngineBackendLayer::getMemoryShapes(const std::vector<MatShape> &inputs,
 
 bool InfEngineBackendLayer::supportBackend(int backendId)
 {
+    CV_LOG_DEBUG(NULL, "InfEngineBackendLayer::supportBackend(" << backendId << ")");
     return backendId == DNN_BACKEND_DEFAULT ||
-           (backendId == DNN_BACKEND_INFERENCE_ENGINE && haveInfEngine());
+           (backendId == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019);
 }
 
 void InfEngineBackendLayer::forward(InputArrayOfArrays inputs, OutputArrayOfArrays outputs,
@@ -1027,7 +1101,18 @@ cv::String getInferenceEngineVPUType()
     static cv::String vpu_type = getInferenceEngineVPUType_();
     return vpu_type;
 }
+
 #else  // HAVE_INF_ENGINE
+
+cv::String getInferenceEngineBackendType()
+{
+    CV_Error(Error::StsNotImplemented, "This OpenCV build doesn't include InferenceEngine support");
+}
+cv::String setInferenceEngineBackendType(const cv::String& newBackendType)
+{
+    CV_UNUSED(newBackendType);
+    CV_Error(Error::StsNotImplemented, "This OpenCV build doesn't include InferenceEngine support");
+}
 cv::String getInferenceEngineVPUType()
 {
     CV_Error(Error::StsNotImplemented, "This OpenCV build doesn't include InferenceEngine support");
