@@ -47,6 +47,7 @@
 
 #include "precomp.hpp"
 #include "opencv2/imgproc/imgproc_c.h"
+#include "opencv2/photo/legacy/constants_c.h"
 
 #undef CV_MAT_ELEM_PTR_FAST
 #define CV_MAT_ELEM_PTR_FAST( mat, row, col, pix_size )  \
@@ -78,6 +79,10 @@ CvHeapElem;
 
 class CvPriorityQueueFloat
 {
+private:
+    CvPriorityQueueFloat(const CvPriorityQueueFloat & ); // copy disabled
+    CvPriorityQueueFloat& operator=(const CvPriorityQueueFloat &); // assign disabled
+
 protected:
     CvHeapElem *mem,*empty,*head,*tail;
     int num,in;
@@ -196,12 +201,14 @@ public:
     }
 };
 
-inline float VectorScalMult(CvPoint2D32f v1,CvPoint2D32f v2) {
+static inline float VectorScalMult(const cv::Point2f& v1, const cv::Point2f& v2)
+{
    return v1.x*v2.x+v1.y*v2.y;
 }
 
-inline float VectorLength(CvPoint2D32f v1) {
-   return v1.x*v1.x+v1.y*v1.y;
+static inline float VectorLength(const cv::Point2f& v1)
+{
+    return v1.x*v1.x+v1.y*v1.y;
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////
@@ -276,7 +283,7 @@ icvCalcFMM(const CvMat *f, CvMat *t, CvPriorityQueueFloat *Heap, bool negate) {
    }
 }
 
-
+template <typename data_type>
 static void
 icvTeleaInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQueueFloat *Heap ) {
    int i = 0, j = 0, ii = 0, jj = 0, k, l, q, color = 0;
@@ -292,7 +299,7 @@ icvTeleaInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQu
             else if(q==1) {i=ii;   j=jj-1;}
             else if(q==2) {i=ii+1; j=jj;}
             else if(q==3) {i=ii;   j=jj+1;}
-            if ((i<=1)||(j<=1)||(i>t->rows-1)||(j>t->cols-1)) continue;
+            if ((i<=0)||(j<=0)||(i>t->rows-1)||(j>t->cols-1)) continue;
 
             if (CV_MAT_ELEM(*f,uchar,i,j)==INSIDE) {
                dist = min4(FastMarching_solve(i-1,j,i,j-1,f,t),
@@ -302,7 +309,7 @@ icvTeleaInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQu
                CV_MAT_ELEM(*t,float,i,j) = dist;
 
                for (color=0; color<=2; color++) {
-                  CvPoint2D32f gradI,gradT,r;
+                  cv::Point2f gradI,gradT,r;
                   float Ia=0,Jx=0,Jy=0,s=1.0e-20f,w,dst,lev,dir,sat;
 
                   if (CV_MAT_ELEM(*f,uchar,i,j+1)!=INSIDE) {
@@ -404,7 +411,7 @@ icvTeleaInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQu
             else if(q==1) {i=ii;   j=jj-1;}
             else if(q==2) {i=ii+1; j=jj;}
             else if(q==3) {i=ii;   j=jj+1;}
-            if ((i<=1)||(j<=1)||(i>t->rows-1)||(j>t->cols-1)) continue;
+            if ((i<=0)||(j<=0)||(i>t->rows-1)||(j>t->cols-1)) continue;
 
             if (CV_MAT_ELEM(*f,uchar,i,j)==INSIDE) {
                dist = min4(FastMarching_solve(i-1,j,i,j-1,f,t),
@@ -414,7 +421,7 @@ icvTeleaInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQu
                CV_MAT_ELEM(*t,float,i,j) = dist;
 
                for (color=0; color<=0; color++) {
-                  CvPoint2D32f gradI,gradT,r;
+                  cv::Point2f gradI,gradT,r;
                   float Ia=0,Jx=0,Jy=0,s=1.0e-20f,w,dst,lev,dir,sat;
 
                   if (CV_MAT_ELEM(*f,uchar,i,j+1)!=INSIDE) {
@@ -462,31 +469,31 @@ icvTeleaInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQu
 
                               if (CV_MAT_ELEM(*f,uchar,k,l+1)!=INSIDE) {
                                  if (CV_MAT_ELEM(*f,uchar,k,l-1)!=INSIDE) {
-                                    gradI.x=(float)((CV_MAT_ELEM(*out,uchar,km,lp+1)-CV_MAT_ELEM(*out,uchar,km,lm-1)))*2.0f;
+                                    gradI.x=(float)((CV_MAT_ELEM(*out,data_type,km,lp+1)-CV_MAT_ELEM(*out,data_type,km,lm-1)))*2.0f;
                                  } else {
-                                    gradI.x=(float)((CV_MAT_ELEM(*out,uchar,km,lp+1)-CV_MAT_ELEM(*out,uchar,km,lm)));
+                                    gradI.x=(float)((CV_MAT_ELEM(*out,data_type,km,lp+1)-CV_MAT_ELEM(*out,data_type,km,lm)));
                                  }
                               } else {
                                  if (CV_MAT_ELEM(*f,uchar,k,l-1)!=INSIDE) {
-                                    gradI.x=(float)((CV_MAT_ELEM(*out,uchar,km,lp)-CV_MAT_ELEM(*out,uchar,km,lm-1)));
+                                    gradI.x=(float)((CV_MAT_ELEM(*out,data_type,km,lp)-CV_MAT_ELEM(*out,data_type,km,lm-1)));
                                  } else {
                                     gradI.x=0;
                                  }
                               }
                               if (CV_MAT_ELEM(*f,uchar,k+1,l)!=INSIDE) {
                                  if (CV_MAT_ELEM(*f,uchar,k-1,l)!=INSIDE) {
-                                    gradI.y=(float)((CV_MAT_ELEM(*out,uchar,kp+1,lm)-CV_MAT_ELEM(*out,uchar,km-1,lm)))*2.0f;
+                                    gradI.y=(float)((CV_MAT_ELEM(*out,data_type,kp+1,lm)-CV_MAT_ELEM(*out,data_type,km-1,lm)))*2.0f;
                                  } else {
-                                    gradI.y=(float)((CV_MAT_ELEM(*out,uchar,kp+1,lm)-CV_MAT_ELEM(*out,uchar,km,lm)));
+                                    gradI.y=(float)((CV_MAT_ELEM(*out,data_type,kp+1,lm)-CV_MAT_ELEM(*out,data_type,km,lm)));
                                  }
                               } else {
                                  if (CV_MAT_ELEM(*f,uchar,k-1,l)!=INSIDE) {
-                                    gradI.y=(float)((CV_MAT_ELEM(*out,uchar,kp,lm)-CV_MAT_ELEM(*out,uchar,km-1,lm)));
+                                    gradI.y=(float)((CV_MAT_ELEM(*out,data_type,kp,lm)-CV_MAT_ELEM(*out,data_type,km-1,lm)));
                                  } else {
                                     gradI.y=0;
                                  }
                               }
-                              Ia += (float)w * (float)(CV_MAT_ELEM(*out,uchar,km,lm));
+                              Ia += (float)w * (float)(CV_MAT_ELEM(*out,data_type,km,lm));
                               Jx -= (float)w * (float)(gradI.x*r.x);
                               Jy -= (float)w * (float)(gradI.y*r.y);
                               s  += w;
@@ -496,7 +503,7 @@ icvTeleaInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQu
                   }
                   sat = (float)((Ia/s+(Jx+Jy)/(sqrt(Jx*Jx+Jy*Jy)+1.0e-20f)+0.5f));
                   {
-                  CV_MAT_ELEM(*out,uchar,i-1,j-1) = cv::saturate_cast<uchar>(sat);
+                  CV_MAT_ELEM(*out,data_type,i-1,j-1) = cv::saturate_cast<data_type>(sat);
                   }
                }
 
@@ -508,7 +515,7 @@ icvTeleaInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQu
    }
 }
 
-
+template <typename data_type>
 static void
 icvNSInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQueueFloat *Heap) {
    int i = 0, j = 0, ii = 0, jj = 0, k, l, q, color = 0;
@@ -524,7 +531,7 @@ icvNSInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQueue
             else if(q==1) {i=ii;   j=jj-1;}
             else if(q==2) {i=ii+1; j=jj;}
             else if(q==3) {i=ii;   j=jj+1;}
-            if ((i<=1)||(j<=1)||(i>t->rows-1)||(j>t->cols-1)) continue;
+            if ((i<=0)||(j<=0)||(i>t->rows-1)||(j>t->cols-1)) continue;
 
             if (CV_MAT_ELEM(*f,uchar,i,j)==INSIDE) {
                dist = min4(FastMarching_solve(i-1,j,i,j-1,f,t),
@@ -534,7 +541,7 @@ icvNSInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQueue
                CV_MAT_ELEM(*t,float,i,j) = dist;
 
                for (color=0; color<=2; color++) {
-                  CvPoint2D32f gradI,r;
+                  cv::Point2f gradI,r;
                   float Ia=0,s=1.0e-20f,w,dst,dir;
 
                   for (k=i-range; k<=i+range; k++) {
@@ -612,7 +619,7 @@ icvNSInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQueue
             else if(q==1) {i=ii;   j=jj-1;}
             else if(q==2) {i=ii+1; j=jj;}
             else if(q==3) {i=ii;   j=jj+1;}
-            if ((i<=1)||(j<=1)||(i>t->rows-1)||(j>t->cols-1)) continue;
+            if ((i<=0)||(j<=0)||(i>t->rows-1)||(j>t->cols-1)) continue;
 
             if (CV_MAT_ELEM(*f,uchar,i,j)==INSIDE) {
                dist = min4(FastMarching_solve(i-1,j,i,j-1,f,t),
@@ -622,7 +629,7 @@ icvNSInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQueue
                CV_MAT_ELEM(*t,float,i,j) = dist;
 
                {
-                  CvPoint2D32f gradI,r;
+                  cv::Point2f gradI,r;
                   float Ia=0,s=1.0e-20f,w,dst,dir;
 
                   for (k=i-range; k<=i+range; k++) {
@@ -639,28 +646,28 @@ icvNSInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQueue
 
                               if (CV_MAT_ELEM(*f,uchar,k+1,l)!=INSIDE) {
                                  if (CV_MAT_ELEM(*f,uchar,k-1,l)!=INSIDE) {
-                                    gradI.x=(float)(abs(CV_MAT_ELEM(*out,uchar,kp+1,lm)-CV_MAT_ELEM(*out,uchar,kp,lm))+
-                                                    abs(CV_MAT_ELEM(*out,uchar,kp,lm)-CV_MAT_ELEM(*out,uchar,km-1,lm)));
+                                    gradI.x=(float)(std::abs(CV_MAT_ELEM(*out,data_type,kp+1,lm)-CV_MAT_ELEM(*out,data_type,kp,lm))+
+                                                    std::abs(CV_MAT_ELEM(*out,data_type,kp,lm)-CV_MAT_ELEM(*out,data_type,km-1,lm)));
                                  } else {
-                                    gradI.x=(float)(abs(CV_MAT_ELEM(*out,uchar,kp+1,lm)-CV_MAT_ELEM(*out,uchar,kp,lm)))*2.0f;
+                                    gradI.x=(float)(std::abs(CV_MAT_ELEM(*out,data_type,kp+1,lm)-CV_MAT_ELEM(*out,data_type,kp,lm)))*2.0f;
                                  }
                               } else {
                                  if (CV_MAT_ELEM(*f,uchar,k-1,l)!=INSIDE) {
-                                    gradI.x=(float)(abs(CV_MAT_ELEM(*out,uchar,kp,lm)-CV_MAT_ELEM(*out,uchar,km-1,lm)))*2.0f;
+                                    gradI.x=(float)(std::abs(CV_MAT_ELEM(*out,data_type,kp,lm)-CV_MAT_ELEM(*out,data_type,km-1,lm)))*2.0f;
                                  } else {
                                     gradI.x=0;
                                  }
                               }
                               if (CV_MAT_ELEM(*f,uchar,k,l+1)!=INSIDE) {
                                  if (CV_MAT_ELEM(*f,uchar,k,l-1)!=INSIDE) {
-                                    gradI.y=(float)(abs(CV_MAT_ELEM(*out,uchar,km,lp+1)-CV_MAT_ELEM(*out,uchar,km,lm))+
-                                                    abs(CV_MAT_ELEM(*out,uchar,km,lm)-CV_MAT_ELEM(*out,uchar,km,lm-1)));
+                                    gradI.y=(float)(std::abs(CV_MAT_ELEM(*out,data_type,km,lp+1)-CV_MAT_ELEM(*out,data_type,km,lm))+
+                                                    std::abs(CV_MAT_ELEM(*out,data_type,km,lm)-CV_MAT_ELEM(*out,data_type,km,lm-1)));
                                  } else {
-                                    gradI.y=(float)(abs(CV_MAT_ELEM(*out,uchar,km,lp+1)-CV_MAT_ELEM(*out,uchar,km,lm)))*2.0f;
+                                    gradI.y=(float)(std::abs(CV_MAT_ELEM(*out,data_type,km,lp+1)-CV_MAT_ELEM(*out,data_type,km,lm)))*2.0f;
                                  }
                               } else {
                                  if (CV_MAT_ELEM(*f,uchar,k,l-1)!=INSIDE) {
-                                    gradI.y=(float)(abs(CV_MAT_ELEM(*out,uchar,km,lm)-CV_MAT_ELEM(*out,uchar,km,lm-1)))*2.0f;
+                                    gradI.y=(float)(std::abs(CV_MAT_ELEM(*out,data_type,km,lm)-CV_MAT_ELEM(*out,data_type,km,lm-1)))*2.0f;
                                  } else {
                                     gradI.y=0;
                                  }
@@ -675,13 +682,13 @@ icvNSInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQueue
                                  dir = (float)fabs(VectorScalMult(r,gradI)/sqrt(VectorLength(r)*VectorLength(gradI)));
                               }
                               w = dst*dir;
-                              Ia += (float)w * (float)(CV_MAT_ELEM(*out,uchar,km,lm));
+                              Ia += (float)w * (float)(CV_MAT_ELEM(*out,data_type,km,lm));
                               s  += w;
                            }
                         }
                      }
                   }
-                  CV_MAT_ELEM(*out,uchar,i-1,j-1) = cv::saturate_cast<uchar>((double)Ia/s);
+                  CV_MAT_ELEM(*out,data_type,i-1,j-1) = cv::saturate_cast<data_type>((double)Ia/s);
                }
 
                CV_MAT_ELEM(*f,uchar,i,j) = BAND;
@@ -716,9 +723,12 @@ icvNSInpaintFMM(const CvMat *f, CvMat *t, CvMat *out, int range, CvPriorityQueue
       }\
    }
 
+namespace cv {
+template<> struct DefaultDeleter<IplConvKernel>{ void operator ()(IplConvKernel* obj) const { cvReleaseStructuringElement(&obj); } };
+}
 
-void
-cvInpaint( const CvArr* _input_img, const CvArr* _inpaint_mask, CvArr* _output_img,
+static void
+icvInpaint( const CvArr* _input_img, const CvArr* _inpaint_mask, CvArr* _output_img,
            double inpaintRange, int flags )
 {
     cv::Ptr<CvMat> mask, band, f, t, out;
@@ -737,11 +747,13 @@ cvInpaint( const CvArr* _input_img, const CvArr* _inpaint_mask, CvArr* _output_i
     if( !CV_ARE_SIZES_EQ(input_img,output_img) || !CV_ARE_SIZES_EQ(input_img,inpaint_mask))
         CV_Error( CV_StsUnmatchedSizes, "All the input and output images must have the same size" );
 
-    if( (CV_MAT_TYPE(input_img->type) != CV_8UC1 &&
+    if( (CV_MAT_TYPE(input_img->type) != CV_8U &&
+         CV_MAT_TYPE(input_img->type) != CV_16U &&
+         CV_MAT_TYPE(input_img->type) != CV_32F &&
         CV_MAT_TYPE(input_img->type) != CV_8UC3) ||
         !CV_ARE_TYPES_EQ(input_img,output_img) )
         CV_Error( CV_StsUnsupportedFormat,
-        "Only 8-bit 1-channel and 3-channel input/output images are supported" );
+        "8-bit, 16-bit unsigned or 32-bit float 1-channel and 8-bit 3-channel input/output images are supported" );
 
     if( CV_MAT_TYPE(inpaint_mask->type) != CV_8UC1 )
         CV_Error( CV_StsUnsupportedFormat, "The mask must be 8-bit 1-channel image" );
@@ -752,11 +764,11 @@ cvInpaint( const CvArr* _input_img, const CvArr* _inpaint_mask, CvArr* _output_i
     ecols = input_img->cols + 2;
     erows = input_img->rows + 2;
 
-    f = cvCreateMat(erows, ecols, CV_8UC1);
-    t = cvCreateMat(erows, ecols, CV_32FC1);
-    band = cvCreateMat(erows, ecols, CV_8UC1);
-    mask = cvCreateMat(erows, ecols, CV_8UC1);
-    el_cross = cvCreateStructuringElementEx(3,3,1,1,CV_SHAPE_CROSS,NULL);
+    f.reset(cvCreateMat(erows, ecols, CV_8UC1));
+    t.reset(cvCreateMat(erows, ecols, CV_32FC1));
+    band.reset(cvCreateMat(erows, ecols, CV_8UC1));
+    mask.reset(cvCreateMat(erows, ecols, CV_8UC1));
+    el_cross.reset(cvCreateStructuringElementEx(3,3,1,1,CV_SHAPE_CROSS,NULL));
 
     cvCopy( input_img, output_img );
     cvSet(mask,cvScalar(KNOWN,0,0,0));
@@ -765,7 +777,7 @@ cvInpaint( const CvArr* _input_img, const CvArr* _inpaint_mask, CvArr* _output_i
     cvSet(f,cvScalar(KNOWN,0,0,0));
     cvSet(t,cvScalar(1.0e6f,0,0,0));
     cvDilate(mask,band,el_cross,1);   // image with narrow band
-    Heap=new CvPriorityQueueFloat;
+    Heap=cv::makePtr<CvPriorityQueueFloat>();
     if (!Heap->Init(band))
         return;
     cvSub(band,mask,band,NULL);
@@ -776,14 +788,14 @@ cvInpaint( const CvArr* _input_img, const CvArr* _inpaint_mask, CvArr* _output_i
     cvSet(f,cvScalar(INSIDE,0,0,0),mask);
     cvSet(t,cvScalar(0,0,0,0),band);
 
-    if( flags == CV_INPAINT_TELEA )
+    if( flags == cv::INPAINT_TELEA )
     {
-        out = cvCreateMat(erows, ecols, CV_8UC1);
-        el_range = cvCreateStructuringElementEx(2*range+1,2*range+1,
-            range,range,CV_SHAPE_RECT,NULL);
+        out.reset(cvCreateMat(erows, ecols, CV_8UC1));
+        el_range.reset(cvCreateStructuringElementEx(2*range+1,2*range+1,
+            range,range,CV_SHAPE_RECT,NULL));
         cvDilate(mask,out,el_range,1);
         cvSub(out,mask,out,NULL);
-        Out=new CvPriorityQueueFloat;
+        Out=cv::makePtr<CvPriorityQueueFloat>();
         if (!Out->Init(out))
             return;
         if (!Out->Add(band))
@@ -791,20 +803,49 @@ cvInpaint( const CvArr* _input_img, const CvArr* _inpaint_mask, CvArr* _output_i
         cvSub(out,band,out,NULL);
         SET_BORDER1_C1(out,uchar,0);
         icvCalcFMM(out,t,Out,true);
-        icvTeleaInpaintFMM(mask,t,output_img,range,Heap);
+        switch(CV_MAT_DEPTH(output_img->type))
+        {
+            case CV_8U:
+                icvTeleaInpaintFMM<uchar>(mask,t,output_img,range,Heap);
+                break;
+            case CV_16U:
+                icvTeleaInpaintFMM<ushort>(mask,t,output_img,range,Heap);
+                break;
+            case CV_32F:
+                icvTeleaInpaintFMM<float>(mask,t,output_img,range,Heap);
+                break;
+            default:
+                CV_Error( cv::Error::StsBadArg, "Unsupportedformat of the input image" );
+        }
     }
-    else if (flags == CV_INPAINT_NS) {
-        icvNSInpaintFMM(mask,t,output_img,range,Heap);
+    else if (flags == cv::INPAINT_NS) {
+        switch(CV_MAT_DEPTH(output_img->type))
+        {
+            case CV_8U:
+                icvNSInpaintFMM<uchar>(mask,t,output_img,range,Heap);
+                break;
+            case CV_16U:
+                icvNSInpaintFMM<ushort>(mask,t,output_img,range,Heap);
+                break;
+            case CV_32F:
+                icvNSInpaintFMM<float>(mask,t,output_img,range,Heap);
+                break;
+            default:
+                CV_Error( cv::Error::StsBadArg, "Unsupported format of the input image" );
+        }
     } else {
-        CV_Error( CV_StsBadArg, "The flags argument must be one of CV_INPAINT_TELEA or CV_INPAINT_NS" );
+        CV_Error( cv::Error::StsBadArg, "The flags argument must be one of CV_INPAINT_TELEA or CV_INPAINT_NS" );
     }
 }
 
 void cv::inpaint( InputArray _src, InputArray _mask, OutputArray _dst,
                   double inpaintRange, int flags )
 {
+    CV_INSTRUMENT_REGION();
+
     Mat src = _src.getMat(), mask = _mask.getMat();
     _dst.create( src.size(), src.type() );
-    CvMat c_src = src, c_mask = mask, c_dst = _dst.getMat();
-    cvInpaint( &c_src, &c_mask, &c_dst, inpaintRange, flags );
+    Mat dst = _dst.getMat();
+    CvMat c_src = cvMat(src), c_mask = cvMat(mask), c_dst = cvMat(dst);
+    icvInpaint( &c_src, &c_mask, &c_dst, inpaintRange, flags );
 }
