@@ -41,15 +41,26 @@
 //M*/
 
 #include "precomp.hpp"
+#include "opencv2/imgproc/detail/gcgraph.hpp"
 #include <map>
 
-using namespace std;
-
 namespace cv {
 namespace detail {
 
-void PairwiseSeamFinder::find(const vector<Mat> &src, const vector<Point> &corners,
-                              vector<Mat> &masks)
+Ptr<SeamFinder> SeamFinder::createDefault(int type)
+{
+    if (type == NO)
+        return makePtr<NoSeamFinder>();
+    if (type == VORONOI_SEAM)
+        return makePtr<VoronoiSeamFinder>();
+    if (type == DP_SEAM)
+        return makePtr<DpSeamFinder>();
+    CV_Error(Error::StsBadArg, "unsupported seam finder method");
+}
+
+
+void PairwiseSeamFinder::find(const std::vector<UMat> &src, const std::vector<Point> &corners,
+                              std::vector<UMat> &masks)
 {
     LOGLN("Finding seams...");
     if (src.size() == 0)
@@ -84,9 +95,14 @@ void PairwiseSeamFinder::run()
     }
 }
 
+void VoronoiSeamFinder::find(const std::vector<UMat> &src, const std::vector<Point> &corners,
+                             std::vector<UMat> &masks)
+{
+    PairwiseSeamFinder::find(src, corners, masks);
+}
 
-void VoronoiSeamFinder::find(const vector<Size> &sizes, const vector<Point> &corners,
-                             vector<Mat> &masks)
+void VoronoiSeamFinder::find(const std::vector<Size> &sizes, const std::vector<Point> &corners,
+                             std::vector<UMat> &masks)
 {
     LOGLN("Finding seams...");
     if (sizes.size() == 0)
@@ -112,7 +128,7 @@ void VoronoiSeamFinder::findInPair(size_t first, size_t second, Rect roi)
     Mat submask2(roi.height + 2 * gap, roi.width + 2 * gap, CV_8U);
 
     Size img1 = sizes_[first], img2 = sizes_[second];
-    Mat mask1 = masks_[first], mask2 = masks_[second];
+    Mat mask1 = masks_[first].getMat(ACCESS_RW), mask2 = masks_[second].getMat(ACCESS_RW);
     Point tl1 = corners_[first], tl2 = corners_[second];
 
     // Cut submasks with some gap
@@ -141,8 +157,8 @@ void VoronoiSeamFinder::findInPair(size_t first, size_t second, Rect roi)
     Mat unique2 = submask2.clone(); unique2.setTo(0, collision);
 
     Mat dist1, dist2;
-    distanceTransform(unique1 == 0, dist1, CV_DIST_L1, 3);
-    distanceTransform(unique2 == 0, dist2, CV_DIST_L1, 3);
+    distanceTransform(unique1 == 0, dist1, DIST_L1, 3);
+    distanceTransform(unique2 == 0, dist2, DIST_L1, 3);
 
     Mat seam = dist1 < dist2;
 
@@ -159,10 +175,30 @@ void VoronoiSeamFinder::findInPair(size_t first, size_t second, Rect roi)
 }
 
 
-DpSeamFinder::DpSeamFinder(CostFunction costFunc) : costFunc_(costFunc) {}
+DpSeamFinder::DpSeamFinder(CostFunction costFunc) : costFunc_(costFunc), ncomps_(0) {}
 
+DpSeamFinder::DpSeamFinder(String costFunc)
+{
+    ncomps_ = 0;
+    if (costFunc == "COLOR")
+        costFunc_ = COLOR;
+    else if (costFunc == "COLOR_GRAD")
+        costFunc_ = COLOR_GRAD;
+    else
+        CV_Error(-1, "Unknown cost function");
+}
 
-void DpSeamFinder::find(const vector<Mat> &src, const vector<Point> &corners, vector<Mat> &masks)
+void DpSeamFinder::setCostFunction(String costFunc)
+{
+    if (costFunc == "COLOR")
+        costFunc_ = COLOR;
+    else if (costFunc == "COLOR_GRAD")
+        costFunc_ = COLOR_GRAD;
+    else
+        CV_Error(-1, "Unknown cost function");
+}
+
+void DpSeamFinder::find(const std::vector<UMat> &src, const std::vector<Point> &corners, std::vector<UMat> &masks)
 {
     LOGLN("Finding seams...");
 #if ENABLE_LOG
@@ -172,19 +208,24 @@ void DpSeamFinder::find(const vector<Mat> &src, const vector<Point> &corners, ve
     if (src.size() == 0)
         return;
 
-    vector<pair<size_t, size_t> > pairs;
+    std::vector<std::pair<size_t, size_t> > pairs;
 
     for (size_t i = 0; i+1 < src.size(); ++i)
         for (size_t j = i+1; j < src.size(); ++j)
-            pairs.push_back(make_pair(i, j));
+            pairs.push_back(std::make_pair(i, j));
 
-    sort(pairs.begin(), pairs.end(), ImagePairLess(src, corners));
-    reverse(pairs.begin(), pairs.end());
+    {
+        std::vector<Mat> _src(src.size());
+        for (size_t i = 0; i < src.size(); ++i) _src[i] = src[i].getMat(ACCESS_READ);
+        sort(pairs.begin(), pairs.end(), ImagePairLess(_src, corners));
+    }
+    std::reverse(pairs.begin(), pairs.end());
 
     for (size_t i = 0; i < pairs.size(); ++i)
     {
         size_t i0 = pairs[i].first, i1 = pairs[i].second;
-        process(src[i0], src[i1], corners[i0], corners[i1], masks[i0], masks[i1]);
+        Mat mask0 = masks[i0].getMat(ACCESS_RW), mask1 = masks[i1].getMat(ACCESS_RW);
+        process(src[i0].getMat(ACCESS_READ), src[i1].getMat(ACCESS_READ), corners[i0], corners[i1], mask0, mask1);
     }
 
     LOGLN("Finding seams, time: " << ((getTickCount() - t) / getTickFrequency()) << " sec");
@@ -195,6 +236,8 @@ void DpSeamFinder::process(
         const Mat &image1, const Mat &image2, Point tl1, Point tl2,
         Mat &mask1, Mat &mask2)
 {
+    CV_INSTRUMENT_REGION();
+
     CV_Assert(image1.size() == mask1.size());
     CV_Assert(image2.size() == mask2.size());
 
@@ -271,11 +314,11 @@ void DpSeamFinder::findComponents()
         for (int x = 0; x < unionSize_.width; ++x)
         {
             if (mask1_(y, x) && mask2_(y, x))
-                labels_(y, x) = numeric_limits<int>::max();
+                labels_(y, x) = std::numeric_limits<int>::max();
             else if (mask1_(y, x))
-                labels_(y, x) = numeric_limits<int>::max()-1;
+                labels_(y, x) = std::numeric_limits<int>::max()-1;
             else if (mask2_(y, x))
-                labels_(y, x) = numeric_limits<int>::max()-2;
+                labels_(y, x) = std::numeric_limits<int>::max()-2;
             else
                 labels_(y, x) = 0;
         }
@@ -285,19 +328,19 @@ void DpSeamFinder::findComponents()
     {
         for (int x = 0; x < unionSize_.width; ++x)
         {
-            if (labels_(y, x) >= numeric_limits<int>::max()-2)
+            if (labels_(y, x) >= std::numeric_limits<int>::max()-2)
             {
-                if (labels_(y, x) == numeric_limits<int>::max())
+                if (labels_(y, x) == std::numeric_limits<int>::max())
                     states_.push_back(INTERS);
-                else if (labels_(y, x) == numeric_limits<int>::max()-1)
+                else if (labels_(y, x) == std::numeric_limits<int>::max()-1)
                     states_.push_back(FIRST);
-                else if (labels_(y, x) == numeric_limits<int>::max()-2)
+                else if (labels_(y, x) == std::numeric_limits<int>::max()-2)
                     states_.push_back(SECOND);
 
                 floodFill(labels_, Point(x, y), ++ncomps_);
                 tls_.push_back(Point(x, y));
                 brs_.push_back(Point(x+1, y+1));
-                contours_.push_back(vector<Point>());
+                contours_.push_back(std::vector<Point>());
             }
 
             if (labels_(y, x))
@@ -325,14 +368,14 @@ void DpSeamFinder::findEdges()
 {
     // find edges between components
 
-    map<pair<int, int>, int> wedges; // weighted edges
+    std::map<std::pair<int, int>, int> wedges; // weighted edges
 
     for (int ci = 0; ci < ncomps_-1; ++ci)
     {
         for (int cj = ci+1; cj < ncomps_; ++cj)
         {
-            wedges[make_pair(ci, cj)] = 0;
-            wedges[make_pair(cj, ci)] = 0;
+            wedges[std::make_pair(ci, cj)] = 0;
+            wedges[std::make_pair(cj, ci)] = 0;
         }
     }
 
@@ -346,26 +389,26 @@ void DpSeamFinder::findEdges()
 
             if (x > 0 && labels_(y, x-1) && labels_(y, x-1) != l)
             {
-                wedges[make_pair(ci, labels_(y, x-1)-1)]++;
-                wedges[make_pair(labels_(y, x-1)-1, ci)]++;
+                wedges[std::make_pair(ci, labels_(y, x-1)-1)]++;
+                wedges[std::make_pair(labels_(y, x-1)-1, ci)]++;
             }
 
             if (y > 0 && labels_(y-1, x) && labels_(y-1, x) != l)
             {
-                wedges[make_pair(ci, labels_(y-1, x)-1)]++;
-                wedges[make_pair(labels_(y-1, x)-1, ci)]++;
+                wedges[std::make_pair(ci, labels_(y-1, x)-1)]++;
+                wedges[std::make_pair(labels_(y-1, x)-1, ci)]++;
             }
 
             if (x < unionSize_.width-1 && labels_(y, x+1) && labels_(y, x+1) != l)
             {
-                wedges[make_pair(ci, labels_(y, x+1)-1)]++;
-                wedges[make_pair(labels_(y, x+1)-1, ci)]++;
+                wedges[std::make_pair(ci, labels_(y, x+1)-1)]++;
+                wedges[std::make_pair(labels_(y, x+1)-1, ci)]++;
             }
 
             if (y < unionSize_.height-1 && labels_(y+1, x) && labels_(y+1, x) != l)
             {
-                wedges[make_pair(ci, labels_(y+1, x)-1)]++;
-                wedges[make_pair(labels_(y+1, x)-1, ci)]++;
+                wedges[std::make_pair(ci, labels_(y+1, x)-1)]++;
+                wedges[std::make_pair(labels_(y+1, x)-1, ci)]++;
             }
         }
     }
@@ -376,11 +419,11 @@ void DpSeamFinder::findEdges()
     {
         for (int cj = ci+1; cj < ncomps_; ++cj)
         {
-            map<pair<int, int>, int>::iterator itr = wedges.find(make_pair(ci, cj));
+            std::map<std::pair<int, int>, int>::iterator itr = wedges.find(std::make_pair(ci, cj));
             if (itr != wedges.end() && itr->second > 0)
                 edges_.insert(itr->first);
 
-            itr = wedges.find(make_pair(cj, ci));
+            itr = wedges.find(std::make_pair(cj, ci));
             if (itr != wedges.end() && itr->second > 0)
                 edges_.insert(itr->first);
         }
@@ -402,7 +445,7 @@ void DpSeamFinder::resolveConflicts(
         int c1 = 0, c2 = 0;
         hasConflict = false;
 
-        for (set<pair<int, int> >::iterator itr = edges_.begin(); itr != edges_.end(); ++itr)
+        for (std::set<std::pair<int, int> >::iterator itr = edges_.begin(); itr != edges_.end(); ++itr)
         {
             c1 = itr->first;
             c2 = itr->second;
@@ -436,7 +479,7 @@ void DpSeamFinder::resolveConflicts(
                 Point p1, p2;
                 if (getSeamTips(c1, c2, p1, p2))
                 {
-                    vector<Point> seam;
+                    std::vector<Point> seam;
                     bool isHorizontalSeam;
 
                     if (estimateSeam(image1, image2, tl1, tl2, c1, p1, p2, seam, isHorizontalSeam))
@@ -456,8 +499,8 @@ void DpSeamFinder::resolveConflicts(
                 int x0 = tls_[c[i]].x, x1 = brs_[c[i]].x;
                 int y0 = tls_[c[i]].y, y1 = brs_[c[i]].y;
 
-                tls_[c[i]] = Point(numeric_limits<int>::max(), numeric_limits<int>::max());
-                brs_[c[i]] = Point(numeric_limits<int>::min(), numeric_limits<int>::min());
+                tls_[c[i]] = Point(std::numeric_limits<int>::max(), std::numeric_limits<int>::max());
+                brs_[c[i]] = Point(std::numeric_limits<int>::min(), std::numeric_limits<int>::min());
                 contours_[c[i]].clear();
 
                 for (int y = y0; y < y1; ++y)
@@ -483,8 +526,8 @@ void DpSeamFinder::resolveConflicts(
 
             // remove edges
 
-            edges_.erase(make_pair(c1, c2));
-            edges_.erase(make_pair(c2, c1));
+            edges_.erase(std::make_pair(c1, c2));
+            edges_.erase(std::make_pair(c2, c1));
         }
     }
 
@@ -524,17 +567,17 @@ void DpSeamFinder::computeGradients(const Mat &image1, const Mat &image2)
     Mat gray;
 
     if (image1.channels() == 3)
-        cvtColor(image1, gray, CV_BGR2GRAY);
+        cvtColor(image1, gray, COLOR_BGR2GRAY);
     else if (image1.channels() == 4)
-        cvtColor(image1, gray, CV_BGRA2GRAY);
+        cvtColor(image1, gray, COLOR_BGRA2GRAY);
 
     Sobel(gray, gradx1_, CV_32F, 1, 0);
     Sobel(gray, grady1_, CV_32F, 0, 1);
 
     if (image2.channels() == 3)
-        cvtColor(image2, gray, CV_BGR2GRAY);
+        cvtColor(image2, gray, COLOR_BGR2GRAY);
     else if (image2.channels() == 4)
-        cvtColor(image2, gray, CV_BGRA2GRAY);
+        cvtColor(image2, gray, COLOR_BGRA2GRAY);
 
     Sobel(gray, gradx2_, CV_32F, 1, 0);
     Sobel(gray, grady2_, CV_32F, 0, 1);
@@ -543,9 +586,9 @@ void DpSeamFinder::computeGradients(const Mat &image1, const Mat &image2)
 
 bool DpSeamFinder::hasOnlyOneNeighbor(int comp)
 {
-    set<pair<int, int> >::iterator begin, end;
-    begin = lower_bound(edges_.begin(), edges_.end(), make_pair(comp, numeric_limits<int>::min()));
-    end = upper_bound(edges_.begin(), edges_.end(), make_pair(comp, numeric_limits<int>::max()));
+    std::set<std::pair<int, int> >::iterator begin, end;
+    begin = lower_bound(edges_.begin(), edges_.end(), std::make_pair(comp, std::numeric_limits<int>::min()));
+    end = upper_bound(edges_.begin(), edges_.end(), std::make_pair(comp, std::numeric_limits<int>::max()));
     return ++begin == end;
 }
 
@@ -579,7 +622,7 @@ bool DpSeamFinder::getSeamTips(int comp1, int comp2, Point &p1, Point &p2)
 
     // find special points
 
-    vector<Point> specialPoints;
+    std::vector<Point> specialPoints;
     int l2 = comp2+1;
 
     for (size_t i = 0; i < contours_[comp1].size(); ++i)
@@ -603,15 +646,15 @@ bool DpSeamFinder::getSeamTips(int comp1, int comp2, Point &p1, Point &p2)
 
     // find clusters
 
-    vector<int> labels;
+    std::vector<int> labels;
     cv::partition(specialPoints, labels, ClosePoints(10));
 
-    int nlabels = *max_element(labels.begin(), labels.end()) + 1;
+    int nlabels = *std::max_element(labels.begin(), labels.end()) + 1;
     if (nlabels < 2)
         return false;
 
-    vector<Point> sum(nlabels);
-    vector<vector<Point> > points(nlabels);
+    std::vector<Point> sum(nlabels);
+    std::vector<std::vector<Point> > points(nlabels);
 
     for (size_t i = 0; i < specialPoints.size(); ++i)
     {
@@ -622,7 +665,7 @@ bool DpSeamFinder::getSeamTips(int comp1, int comp2, Point &p1, Point &p2)
     // select two most distant clusters
 
     int idx[2] = {-1,-1};
-    double maxDist = -numeric_limits<double>::max();
+    double maxDist = -std::numeric_limits<double>::max();
 
     for (int i = 0; i < nlabels-1; ++i)
     {
@@ -630,7 +673,7 @@ bool DpSeamFinder::getSeamTips(int comp1, int comp2, Point &p1, Point &p2)
         {
             double size1 = static_cast<double>(points[i].size()), size2 = static_cast<double>(points[j].size());
             double cx1 = cvRound(sum[i].x / size1), cy1 = cvRound(sum[i].y / size1);
-            double cx2 = cvRound(sum[j].x / size2), cy2 = cvRound(sum[j].y / size1);
+            double cx2 = cvRound(sum[j].x / size2), cy2 = cvRound(sum[j].y / size2);
 
             double dist = (cx1 - cx2) * (cx1 - cx2) + (cy1 - cy2) * (cy1 - cy2);
             if (dist > maxDist)
@@ -653,7 +696,7 @@ bool DpSeamFinder::getSeamTips(int comp1, int comp2, Point &p1, Point &p2)
         double cy = cvRound(sum[idx[i]].y / size);
 
         size_t closest = points[idx[i]].size();
-        double minDist = numeric_limits<double>::max();
+        double minDist = std::numeric_limits<double>::max();
 
         for (size_t j = 0; j < points[idx[i]].size(); ++j)
         {
@@ -718,7 +761,7 @@ void DpSeamFinder::computeCosts(
     else if (image1.type() == CV_8UC4 && image2.type() == CV_8UC4)
         diff = diffL2Square4<uchar>;
     else
-        CV_Error(CV_StsBadArg, "both images must have CV_32FC3(4) or CV_8UC3(4) type");
+        CV_Error(Error::StsBadArg, "both images must have CV_32FC3(4) or CV_8UC3(4) type");
 
     int l = comp+1;
     Rect roi(tls_[comp], brs_[comp]);
@@ -781,7 +824,7 @@ void DpSeamFinder::computeCosts(
 
 bool DpSeamFinder::estimateSeam(
         const Mat &image1, const Mat &image2, Point tl1, Point tl2, int comp,
-        Point p1, Point p2, vector<Point> &seam, bool &isHorizontal)
+        Point p1, Point p2, std::vector<Point> &seam, bool &isHorizontal)
 {
     CV_Assert(states_[comp] & INTERS);
 
@@ -822,7 +865,7 @@ bool DpSeamFinder::estimateSeam(
     cost(src) = 0.f;
 
     int nsteps;
-    pair<float, int> steps[3];
+    std::pair<float, int> steps[3];
 
     if (isHorizontal)
     {
@@ -837,16 +880,16 @@ bool DpSeamFinder::estimateSeam(
                 if (labels_(y + roi.y, x + roi.x) == l)
                 {
                     if (reachable(y, x-1))
-                        steps[nsteps++] = make_pair(cost(y, x-1) + costH(y, x-1), 1);
+                        steps[nsteps++] = std::make_pair(cost(y, x-1) + costH(y, x-1), 1);
                     if (y > 0 && reachable(y-1, x-1))
-                        steps[nsteps++] = make_pair(cost(y-1, x-1) + costH(y-1, x-1) + costV(y-1, x), 2);
+                        steps[nsteps++] = std::make_pair(cost(y-1, x-1) + costH(y-1, x-1) + costV(y-1, x), 2);
                     if (y < roi.height-1 && reachable(y+1, x-1))
-                        steps[nsteps++] = make_pair(cost(y+1, x-1) + costH(y+1, x-1) + costV(y, x), 3);
+                        steps[nsteps++] = std::make_pair(cost(y+1, x-1) + costH(y+1, x-1) + costV(y, x), 3);
                 }
 
                 if (nsteps)
                 {
-                    pair<float, int> opt = *min_element(steps, steps + nsteps);
+                    std::pair<float, int> opt = *min_element(steps, steps + nsteps);
                     cost(y, x) = opt.first;
                     control(y, x) = (uchar)opt.second;
                     reachable(y, x) = 255;
@@ -867,16 +910,16 @@ bool DpSeamFinder::estimateSeam(
                 if (labels_(y + roi.y, x + roi.x) == l)
                 {
                     if (reachable(y-1, x))
-                        steps[nsteps++] = make_pair(cost(y-1, x) + costV(y-1, x), 1);
+                        steps[nsteps++] = std::make_pair(cost(y-1, x) + costV(y-1, x), 1);
                     if (x > 0 && reachable(y-1, x-1))
-                        steps[nsteps++] = make_pair(cost(y-1, x-1) + costV(y-1, x-1) + costH(y, x-1), 2);
+                        steps[nsteps++] = std::make_pair(cost(y-1, x-1) + costV(y-1, x-1) + costH(y, x-1), 2);
                     if (x < roi.width-1 && reachable(y-1, x+1))
-                        steps[nsteps++] = make_pair(cost(y-1, x+1) + costV(y-1, x+1) + costH(y, x), 3);
+                        steps[nsteps++] = std::make_pair(cost(y-1, x+1) + costV(y-1, x+1) + costH(y, x), 3);
                 }
 
                 if (nsteps)
                 {
-                    pair<float, int> opt = *min_element(steps, steps + nsteps);
+                    std::pair<float, int> opt = *min_element(steps, steps + nsteps);
                     cost(y, x) = opt.first;
                     control(y, x) = (uchar)opt.second;
                     reachable(y, x) = 255;
@@ -914,7 +957,7 @@ bool DpSeamFinder::estimateSeam(
     }
 
     if (!swapped)
-        reverse(seam.begin(), seam.end());
+        std::reverse(seam.begin(), seam.end());
 
     CV_Assert(seam.front() == p1);
     CV_Assert(seam.back() == p2);
@@ -923,7 +966,7 @@ bool DpSeamFinder::estimateSeam(
 
 
 void DpSeamFinder::updateLabelsUsingSeam(
-        int comp1, int comp2, const vector<Point> &seam, bool isHorizontalSeam)
+        int comp1, int comp2, const std::vector<Point> &seam, bool isHorizontalSeam)
 {
     Mat_<int> mask = Mat::zeros(brs_[comp1].y - tls_[comp1].y,
                                 brs_[comp1].x - tls_[comp1].x, CV_32S);
@@ -1001,13 +1044,13 @@ void DpSeamFinder::updateLabelsUsingSeam(
     // find new components connected with the second component and
     // with other components except the ones we are working with
 
-    map<int, int> connect2;
-    map<int, int> connectOther;
+    std::map<int, int> connect2;
+    std::map<int, int> connectOther;
 
     for (int i = 1; i <= ncomps; ++i)
     {
-        connect2.insert(make_pair(i, 0));
-        connectOther.insert(make_pair(i, 0));
+        connect2.insert(std::make_pair(i, 0));
+        connectOther.insert(std::make_pair(i, 0));
     }
 
     for (size_t i = 0; i < contours_[comp1].size(); ++i)
@@ -1032,12 +1075,21 @@ void DpSeamFinder::updateLabelsUsingSeam(
         }
     }
 
-    vector<int> isAdjComp(ncomps + 1, 0);
+    std::vector<int> isAdjComp(ncomps + 1, 0);
 
-    for (map<int, int>::iterator itr = connect2.begin(); itr != connect2.end(); ++itr)
+    for (std::map<int, int>::iterator itr = connect2.begin(); itr != connect2.end(); ++itr)
     {
         double len = static_cast<double>(contours_[comp1].size());
-        isAdjComp[itr->first] = itr->second / len > 0.05 && connectOther.find(itr->first)->second / len < 0.1;
+        int res = 0;
+        if (itr->second / len > 0.05)
+        {
+            std::map<int, int>::const_iterator sub = connectOther.find(itr->first);
+            if (sub != connectOther.end() && (sub->second / len < 0.1))
+            {
+                res = 1;
+            }
+        }
+        isAdjComp[itr->first] = res;
     }
 
     // update labels
@@ -1049,7 +1101,7 @@ void DpSeamFinder::updateLabelsUsingSeam(
 }
 
 
-class GraphCutSeamFinder::Impl : public PairwiseSeamFinder
+class GraphCutSeamFinder::Impl CV_FINAL : public PairwiseSeamFinder
 {
 public:
     Impl(int cost_type, float terminal_cost, float bad_region_penalty)
@@ -1057,8 +1109,8 @@ public:
 
     ~Impl() {}
 
-    void find(const vector<Mat> &src, const vector<Point> &corners, vector<Mat> &masks);
-    void findInPair(size_t first, size_t second, Rect roi);
+    void find(const std::vector<UMat> &src, const std::vector<Point> &corners, std::vector<UMat> &masks) CV_OVERRIDE;
+    void findInPair(size_t first, size_t second, Rect roi) CV_OVERRIDE;
 
 private:
     void setGraphWeightsColor(const Mat &img1, const Mat &img2,
@@ -1067,15 +1119,15 @@ private:
                                   const Mat &dy1, const Mat &dy2, const Mat &mask1, const Mat &mask2,
                                   GCGraph<float> &graph);
 
-    vector<Mat> dx_, dy_;
+    std::vector<Mat> dx_, dy_;
     int cost_type_;
     float terminal_cost_;
     float bad_region_penalty_;
 };
 
 
-void GraphCutSeamFinder::Impl::find(const vector<Mat> &src, const vector<Point> &corners,
-                                    vector<Mat> &masks)
+void GraphCutSeamFinder::Impl::find(const std::vector<UMat> &src, const std::vector<Point> &corners,
+                                    std::vector<UMat> &masks)
 {
     // Compute gradients
     dx_.resize(src.size());
@@ -1209,10 +1261,10 @@ void GraphCutSeamFinder::Impl::setGraphWeightsColorGrad(
 
 void GraphCutSeamFinder::Impl::findInPair(size_t first, size_t second, Rect roi)
 {
-    Mat img1 = images_[first], img2 = images_[second];
+    Mat img1 = images_[first].getMat(ACCESS_READ), img2 = images_[second].getMat(ACCESS_READ);
     Mat dx1 = dx_[first], dx2 = dx_[second];
     Mat dy1 = dy_[first], dy2 = dy_[second];
-    Mat mask1 = masks_[first], mask2 = masks_[second];
+    Mat mask1 = masks_[first].getMat(ACCESS_RW), mask2 = masks_[second].getMat(ACCESS_RW);
     Point tl1 = corners_[first], tl2 = corners_[second];
 
     const int gap = 10;
@@ -1281,7 +1333,7 @@ void GraphCutSeamFinder::Impl::findInPair(size_t first, size_t second, Rect roi)
                                  submask1, submask2, graph);
         break;
     default:
-        CV_Error(CV_StsBadArg, "unsupported pixel similarity measure");
+        CV_Error(Error::StsBadArg, "unsupported pixel similarity measure");
     }
 
     graph.maxFlow();
@@ -1304,6 +1356,19 @@ void GraphCutSeamFinder::Impl::findInPair(size_t first, size_t second, Rect roi)
     }
 }
 
+GraphCutSeamFinder::GraphCutSeamFinder(String cost_type, float terminal_cost, float bad_region_penalty)
+{
+    CostType t;
+    if (cost_type == "COST_COLOR")
+        t = COST_COLOR;
+    else if (cost_type == "COST_COLOR_GRAD")
+        t = COST_COLOR_GRAD;
+    else
+        CV_Error(Error::StsBadFunc, "Unknown cost type function");
+    impl_ = new Impl(t, terminal_cost, bad_region_penalty);
+}
+
+
 
 GraphCutSeamFinder::GraphCutSeamFinder(int cost_type, float terminal_cost, float bad_region_penalty)
     : impl_(new Impl(cost_type, terminal_cost, bad_region_penalty)) {}
@@ -1311,16 +1376,16 @@ GraphCutSeamFinder::GraphCutSeamFinder(int cost_type, float terminal_cost, float
 GraphCutSeamFinder::~GraphCutSeamFinder() {}
 
 
-void GraphCutSeamFinder::find(const vector<Mat> &src, const vector<Point> &corners,
-                              vector<Mat> &masks)
+void GraphCutSeamFinder::find(const std::vector<UMat> &src, const std::vector<Point> &corners,
+                              std::vector<UMat> &masks)
 {
     impl_->find(src, corners, masks);
 }
 
 
-#if defined(HAVE_OPENCV_GPU) && !defined(DYNAMIC_CUDA_SUPPORT)
-void GraphCutSeamFinderGpu::find(const vector<Mat> &src, const vector<Point> &corners,
-                                 vector<Mat> &masks)
+#ifdef HAVE_OPENCV_CUDALEGACY
+void GraphCutSeamFinderGpu::find(const std::vector<UMat> &src, const std::vector<Point> &corners,
+                                 std::vector<UMat> &masks)
 {
     // Compute gradients
     dx_.resize(src.size());
@@ -1352,10 +1417,10 @@ void GraphCutSeamFinderGpu::find(const vector<Mat> &src, const vector<Point> &co
 
 void GraphCutSeamFinderGpu::findInPair(size_t first, size_t second, Rect roi)
 {
-    Mat img1 = images_[first], img2 = images_[second];
+    Mat img1 = images_[first].getMat(ACCESS_READ), img2 = images_[second].getMat(ACCESS_READ);
     Mat dx1 = dx_[first], dx2 = dx_[second];
     Mat dy1 = dy_[first], dy2 = dy_[second];
-    Mat mask1 = masks_[first], mask2 = masks_[second];
+    Mat mask1 = masks_[first].getMat(ACCESS_READ), mask2 = masks_[second].getMat(ACCESS_READ);
     Point tl1 = corners_[first], tl2 = corners_[second];
 
     const int gap = 10;
@@ -1422,17 +1487,17 @@ void GraphCutSeamFinderGpu::findInPair(size_t first, size_t second, Rect roi)
                                  submask1, submask2, terminals, leftT, rightT, top, bottom);
         break;
     default:
-        CV_Error(CV_StsBadArg, "unsupported pixel similarity measure");
+        CV_Error(Error::StsBadArg, "unsupported pixel similarity measure");
     }
 
-    gpu::GpuMat terminals_d(terminals);
-    gpu::GpuMat leftT_d(leftT);
-    gpu::GpuMat rightT_d(rightT);
-    gpu::GpuMat top_d(top);
-    gpu::GpuMat bottom_d(bottom);
-    gpu::GpuMat labels_d, buf_d;
+    cuda::GpuMat terminals_d(terminals);
+    cuda::GpuMat leftT_d(leftT);
+    cuda::GpuMat rightT_d(rightT);
+    cuda::GpuMat top_d(top);
+    cuda::GpuMat bottom_d(bottom);
+    cuda::GpuMat labels_d, buf_d;
 
-    gpu::graphcut(terminals_d, leftT_d, rightT_d, top_d, bottom_d, labels_d, buf_d);
+    cuda::graphcut(terminals_d, leftT_d, rightT_d, top_d, bottom_d, labels_d, buf_d);
 
     Mat_<uchar> labels = (Mat)labels_d;
     for (int y = 0; y < roi.height; ++y)
@@ -1642,62 +1707,6 @@ void GraphCutSeamFinderGpu::setGraphWeightsColorGrad(
         }
     }
 }
-#else
-void GraphCutSeamFinderGpu::find(const vector<Mat> &src, const vector<Point> &corners,
-                                 vector<Mat> &masks)
-{
-    (void)src;
-    (void)corners;
-    (void)masks;
-    CV_Error(CV_StsNotImplemented, "CUDA optimization is unavailable");
-}
-
-
-void GraphCutSeamFinderGpu::findInPair(size_t first, size_t second, Rect roi)
-{
-    (void)first;
-    (void)second;
-    (void)roi;
-    CV_Error(CV_StsNotImplemented, "CUDA optimization is unavailable");
-}
-
-
-void GraphCutSeamFinderGpu::setGraphWeightsColor(const Mat &img1, const Mat &img2, const Mat &mask1, const Mat &mask2,
-                                                 Mat &terminals, Mat &leftT, Mat &rightT, Mat &top, Mat &bottom)
-{
-    (void)img1;
-    (void)img2;
-    (void)mask1;
-    (void)mask2;
-    (void)terminals;
-    (void)leftT;
-    (void)rightT;
-    (void)top;
-    (void)bottom;
-    CV_Error(CV_StsNotImplemented, "CUDA optimization is unavailable");
-}
-
-
-void GraphCutSeamFinderGpu::setGraphWeightsColorGrad(
-        const Mat &img1, const Mat &img2, const Mat &dx1, const Mat &dx2,
-        const Mat &dy1, const Mat &dy2, const Mat &mask1, const Mat &mask2,
-        Mat &terminals, Mat &leftT, Mat &rightT, Mat &top, Mat &bottom)
-{
-    (void)img1;
-    (void)img2;
-    (void)dx1;
-    (void)dx2;
-    (void)dy1;
-    (void)dy2;
-    (void)mask1;
-    (void)mask2;
-    (void)terminals;
-    (void)leftT;
-    (void)rightT;
-    (void)top;
-    (void)bottom;
-    CV_Error(CV_StsNotImplemented, "CUDA optimization is unavailable");
-}
 #endif
 
 } // namespace detail
